@@ -56,32 +56,28 @@ class WireType(enum.IntEnum):
     FIXED_32 = 5
 
 
+def _validate_field_number(number):
+    if number not in range(1, 2 ** 29) or number in range(19_000, 20_000):
+        raise FieldValidationError(
+            'Field number {} is invalid. '
+            'A valid field number should be in range [1..2 ** 29 -1] '
+            'excluding range [19_000..19_999] '
+            'which is reserved for the protobuf implementation'
+        )
+
+
 class Field(ABC):
-    wire_type: WireType  # should be provided by subclasses
+    wire_type: WireType  # provided by subclasses
+    number: int
+    header: bytes
 
-    def __init__(self, *, number: int, default=None, required: bool = False):
-        self._validate_field_number(number)
-
-        self._number = number
-        self._required = required
-        self._header = encode_varint(self.number << 3 | self.wire_type)
-        self._default = default
-
-        # the name of the field is set by message because it's known in the message
+    def __init__(self, *, number: int):
+        _validate_field_number(number)
+        self.number = number
+        self.header = encode_varint(self.number << 3 | self.wire_type)
+        # name is set when adding the field to the message
         self.name: Optional[str] = None
 
-    @property
-    def number(self):
-        return self._number
-
-    @property
-    def default(self):
-        return self._default
-
-    @property
-    def required(self):
-        return self._required
-
     @abstractmethod
     def encode_value(self, value) -> bytes:
         raise NotImplementedError()
@@ -100,33 +96,38 @@ class Field(ABC):
     def validate_value(self, value):
         raise NotImplementedError()
 
-    @property
-    def header(self) -> bytes:
-        return self._header
+
+class PrimitiveField(Field, ABC):
+    def __init__(
+        self,
+        *,
+        number: int,
+        default=None,
+        required=False,
+    ):
+        super().__init__(number=number)
+        self.default = default
+        self.required = required
 
     @classmethod
-    def as_repeated(cls, *, number: int, packed: bool = False) -> 'Repeated':
+    def as_repeated(
+        cls,
+        *,
+        number: int,
+        packed: bool = False,
+    ) -> 'Repeated':
         return Repeated(
             of_type=cls,
             number=number,
             packed=packed,
         )
 
-    @staticmethod
-    def _validate_field_number(number):
-        if number not in range(1, 2 ** 29) or number in range(19_000, 20_000):
-            raise FieldValidationError(
-                'Field number {} is invalid. '
-                'A valid field number should be in range [1..2 ** 29 -1] '
-                'excluding range [19_000..19_999] '
-                'which is reserved for the protobuf implementation'
-            )
-
 
 class BaseRepeatedStrategy(ABC):
     def __init__(self, field: Field, number: int):
-        self._field = field
-        self._number = number
+        self.field = field
+        self.number = number
+        self.header = encode_varint(self.field.number << 3 | WireType.LENGTH)
 
     @abstractmethod
     def encode(self, values: list) -> bytes:
@@ -147,10 +148,10 @@ class PackedRepeatedStrategy(BaseRepeatedStrategy):
         data = bytearray()
 
         for item in values:
-            data += self._field.encode_value(item)
+            data += self.field.encode_value(item)
 
         return (
-            encode_varint(self._field.number << 3 | WireType.LENGTH) +
+            self.header +
             encode_bytes(data)
         )
 
@@ -169,7 +170,7 @@ class PackedRepeatedStrategy(BaseRepeatedStrategy):
         stream = io.BytesIO(data)
 
         while stream.seek(0, io.SEEK_CUR) < length:
-            items.append(self._field.decode(stream))
+            items.append(self.field.decode(stream))
 
         return items
 
@@ -177,7 +178,7 @@ class PackedRepeatedStrategy(BaseRepeatedStrategy):
 class UnpackedRepeatedStrategy(BaseRepeatedStrategy):
     @property
     def wire_type(self):
-        return self._field.wire_type
+        return self.field.wire_type
 
     def encode(self, values: list) -> bytes:
         if not values:
@@ -186,7 +187,7 @@ class UnpackedRepeatedStrategy(BaseRepeatedStrategy):
         data = bytearray()
 
         for item in values:
-            data += self._field.header + self._field.encode_value(item)
+            data += self.field.header + self.field.encode_value(item)
 
         return data
 
@@ -198,7 +199,7 @@ class UnpackedRepeatedStrategy(BaseRepeatedStrategy):
 
         That's why we can only read exactly one item at a time
         """
-        return self._field.decode(stream)
+        return self.field.decode(stream)
 
 
 class Repeated(Field):
@@ -216,51 +217,33 @@ class Repeated(Field):
                 raise FieldValidationError(
                     f'Field of type {of_type.__name__!r}. Packed repeated fields only support primitive types'
                 )
-            self._field = MessageField(of_type, number=number)
+            self.field = MessageField(of_type, number=number)
         elif issubclass(of_type, enum.IntEnum):
-            self._field = EnumField(of_type, number=number)
+            self.field = EnumField(of_type, number=number)
         else:
-            self._field = of_type(number=number)
+            self.field = of_type(number=number)
 
-        self._of_type = of_type
-        self._packed: bool = packed
-        self._strategy: BaseRepeatedStrategy
+        self.of_type = of_type
+        self.packed: bool = packed
+        self.strategy: BaseRepeatedStrategy
 
         if packed:
-            self._strategy = PackedRepeatedStrategy(self._field, number)
+            self.strategy = PackedRepeatedStrategy(self.field, number)
         else:
-            self._strategy = UnpackedRepeatedStrategy(self._field, number)
+            self.strategy = UnpackedRepeatedStrategy(self.field, number)
 
-        self.wire_type = self._strategy.wire_type
+        self.wire_type = self.strategy.wire_type
 
         super().__init__(number=number)
 
-    @classmethod
-    def as_repeated(cls, *, number: int, packed: bool = False) -> 'Repeated':
-        raise Exception(
-            'Cannot created repeated from repeated'
-        )
-
-    @property
-    def field(self):
-        return self._field
-
-    @property
-    def of_type(self):
-        return self._of_type
-
-    @property
-    def packed(self):
-        return self._packed
-
     def encode(self, values: list) -> bytes:
-        return self._strategy.encode(values)
+        return self.strategy.encode(values)
 
     def encode_value(self, values: list) -> bytes:
-        return self._strategy.encode(values)
+        return self.strategy.encode(values)
 
     def decode(self, stream: IO) -> list:
-        return self._strategy.decode(stream)
+        return self.strategy.decode(stream)
 
     def validate_value(self, values: Iterable):
         if not isinstance(values, Iterable):
@@ -269,10 +252,10 @@ class Repeated(Field):
             )
 
         for value in values:
-            self._field.validate_value(value)
+            self.field.validate_value(value)
 
 
-class Int(Field):
+class Int(PrimitiveField):
     min_value: int
     max_value: int
     wire_type = WireType.VARINT
@@ -360,7 +343,7 @@ class SInt64(SInt):
         return encode_varint(zig_zag_value)
 
 
-class Bytes(Field):
+class Bytes(PrimitiveField):
     wire_type = WireType.LENGTH
 
     def encode_value(self, value: bytes) -> bytes:
@@ -377,7 +360,7 @@ class Bytes(Field):
             )
 
 
-class String(Field):
+class String(PrimitiveField):
     wire_type = WireType.LENGTH
 
     def encode_value(self, value: str) -> bytes:
@@ -396,7 +379,7 @@ class String(Field):
             )
 
 
-class Bool(Field):
+class Bool(PrimitiveField):
     wire_type = WireType.VARINT
 
     def encode_value(self, value: bool) -> bytes:
@@ -414,7 +397,20 @@ class Bool(Field):
             )
 
 
-class EnumField(Field):
+def _validate_py_enum(py_enum: Type[enum.IntEnum]):
+    if not issubclass(py_enum, enum.IntEnum):
+        raise FieldValidationError(
+            f'Enum field should be a subclass of IntEnum, got {py_enum!r} instead'
+        )
+
+    for variant in py_enum:
+        if not -2 ** 31 <= variant.value <= 2 ** 31 - 1:
+            raise FieldValidationError(
+                f'Invalid enum field variant {variant.value}. Should be in range [-2 ** 31 .. 2 ** 31 - 1]'
+            )
+
+
+class EnumField(PrimitiveField):
     wire_type = WireType.VARINT
 
     def __init__(
@@ -425,7 +421,7 @@ class EnumField(Field):
         default=None,
         required=False,
     ):
-        self._validate_py_enum(py_enum)
+        _validate_py_enum(py_enum)
 
         super().__init__(
             number=number,
@@ -433,7 +429,7 @@ class EnumField(Field):
             required=required,
         )
 
-        self._py_enum: Type[enum.IntEnum] = py_enum
+        self.py_enum: Type[enum.IntEnum] = py_enum
 
     def encode_value(self, value: int) -> bytes:
         return encode_varint(value)
@@ -443,32 +439,19 @@ class EnumField(Field):
 
         # Specification: omit value that's not in the enum's variants
         try:
-            return self._py_enum(value)
+            return self.py_enum(value)
         except ValueError:
             return None
 
     def validate_value(self, value: int):
         try:
-            self._py_enum(value)
+            self.py_enum(value)
         except ValueError:
             raise ValueError(
-                f'Expected an enum value of {repr([x for x in self._py_enum])}, '
+                f'Expected an enum value of {repr([x for x in self.py_enum])}, '
                 f'got {value} instead'
             )
 
-    @staticmethod
-    def _validate_py_enum(py_enum: Type[enum.IntEnum]):
-        if not issubclass(py_enum, enum.IntEnum):
-            raise FieldValidationError(
-                f'Enum field should be a subclass of IntEnum, got {py_enum!r} instead'
-            )
-
-        for variant in py_enum:
-            if not -2 ** 31 <= variant.value <= 2 ** 31 - 1:
-                raise FieldValidationError(
-                    f'Invalid enum field variant {variant.value}. Should be in range [-2 ** 31 .. 2 ** 31 - 1]'
-                )
-
 
 class Fixed32(Int):
     min_value = MIN_UINT32
@@ -546,7 +529,7 @@ class SFixed64(Int):
         return struct.unpack('<q', data)[0]
 
 
-class Float(Field):
+class Float(PrimitiveField):
     max_value = MAX_FLOAT
     wire_type = WireType.FIXED_32
 
@@ -623,8 +606,8 @@ class MapField(Field):
 
     def __init__(
         self,
-        key: Type[Field],
-        value: Union[Type[Field], Type[enum.IntEnum], Type['Message']],
+        key: Union[Type[Int], Type[String]],
+        value: Union[Type[PrimitiveField], Type[enum.IntEnum], Type['Message']],
         *,
         number: int
     ):
@@ -650,24 +633,16 @@ class MapField(Field):
             key = key_field
             value = value_field
 
-        self._dict_entry = _DictEntry
-        self._key_field = key_field
-        self._value_field = value_field
-
-    @property
-    def key_field(self) -> Field:
-        return self._key_field
-
-    @property
-    def value_field(self) -> Field:
-        return self._value_field
+        self.dict_entry = _DictEntry
+        self.key_field = key_field
+        self.value_field = value_field
 
     def encode_value(self, value: Dict) -> bytes:
         buffer = bytearray()
 
         for key, value in value.items():
             buffer += self.header + encode_bytes(
-                self._dict_entry(
+                self.dict_entry(
                     key=key,
                     value=value
                 ).to_bytes()
@@ -678,7 +653,7 @@ class MapField(Field):
     encode = encode_value
 
     def decode(self, stream: BinaryIO) -> Tuple:
-        entry = self._dict_entry.from_bytes(
+        entry = self.dict_entry.from_bytes(
             decode_bytes(stream)
         )
         return entry.key, entry.value
@@ -698,15 +673,11 @@ class MessageField(Field):
         of_type,
         *,
         number: int,
-        default=None,
-        required=False,
+        required: bool = False
     ):
-        super().__init__(
-            number=number,
-            default=default,
-            required=required,
-        )
-        self._of_type = of_type
+        super().__init__(number=number)
+        self.of_type: Type['Message'] = of_type
+        self.required: bool = required
 
     def encode_value(self, value) -> bytes:
         encoded_message = value.to_bytes()
@@ -715,27 +686,23 @@ class MessageField(Field):
     def decode(self, stream: IO):
         length = decode_varint(stream)
         message_stream = io.BytesIO(stream.read(length))
-        return self._of_type.from_stream(message_stream)
+        return self.of_type.from_stream(message_stream)
 
     def validate_value(self, value):
-        if not isinstance(value, self._of_type):
+        if not isinstance(value, self.of_type):
             raise ValueError(
-                f'Expected a value of type {self._of_type.__name__!r}'
+                f'Expected a value of type {self.of_type.__name__!r}'
                 f', got value of type {type(value).__name__!r} instead'
             )
 
 
 class OneOf:
     def __init__(self, *args: str):
-        self._fields: List[str] = list(args)
+        self.fields: List[str] = list(args)
 
         # name of one_of is set in the message that contains the one_of
         self.name: Optional[str] = None
 
-    @property
-    def fields(self):
-        return self._fields
-
 
 one_of = OneOf
 
